# Generated by Django 2.2.8 on 2020-01-20 01:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('structures', '0005_notification_created'),
    ]

    operations = [
        migrations.CreateModel(
            name='StructureTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='name of the tag - must be unique', max_length=255, unique=True)),
                ('description', models.TextField(blank=True, default=None, help_text='description for this tag', null=True)),
                ('style', models.CharField(blank=True, choices=[('default', 'grey'), ('primary', 'dark blue'), ('success', 'green'), ('info', 'light blue'), ('warning', 'yellow'), ('danger', 'red')], default='default', help_text='color style of tag', max_length=16)),
                ('is_default', models.BooleanField(default=False, help_text='if true this tag will automatically added to new structures')),
            ],
        ),
        migrations.AddField(
            model_name='structure',
            name='tags',
            field=models.ManyToManyField(blank=True, default=None, help_text='list of tags for this structure', to='structures.StructureTag'),
        ),
    ]
