{% extends 'allianceauth/base.html' %}
{% load i18n %}

{% block page_title %}{% trans "Alliance Structures" %}{% endblock %}

{% block content %}
    <div class="col-lg-12">        
        <h2 class="text-center">{{ page_title }}</h2>
        <div class="col-lg-12 container">            
            {% block details %}{% endblock %}
        </div>
    </div>
{% endblock %}
