# Generated by Django 2.2.11 on 2020-03-31 12:26

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('structures', '0008_auto_20200209_1638'),
    ]

    operations = [
        migrations.AddField(
            model_name='evecategory',
            name='last_updated',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='When this object was last updated from ESI', null=True),
        ),
        migrations.AddField(
            model_name='evecategory',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='evecategory',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='evecategory',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='evecategory',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='eveconstellation',
            name='last_updated',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='When this object was last updated from ESI', null=True),
        ),
        migrations.AddField(
            model_name='eveconstellation',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='eveconstellation',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='eveconstellation',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='eveconstellation',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='evegroup',
            name='last_updated',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='When this object was last updated from ESI', null=True),
        ),
        migrations.AddField(
            model_name='evegroup',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='evegroup',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='evegroup',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='evegroup',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='evemoon',
            name='last_updated',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='When this object was last updated from ESI', null=True),
        ),
        migrations.AddField(
            model_name='evemoon',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='evemoon',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='evemoon',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='evemoon',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='eveplanet',
            name='last_updated',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='When this object was last updated from ESI', null=True),
        ),
        migrations.AddField(
            model_name='eveplanet',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='eveplanet',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='eveplanet',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='eveplanet',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='everegion',
            name='last_updated',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='When this object was last updated from ESI', null=True),
        ),
        migrations.AddField(
            model_name='everegion',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='everegion',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='everegion',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='everegion',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='evesolarsystem',
            name='last_updated',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='When this object was last updated from ESI', null=True),
        ),
        migrations.AddField(
            model_name='evesolarsystem',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='evesolarsystem',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='evesolarsystem',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='evesolarsystem',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='evetype',
            name='last_updated',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='When this object was last updated from ESI', null=True),
        ),
        migrations.AddField(
            model_name='evetype',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='evetype',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='evetype',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='evetype',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='structureservice',
            name='name_de',
            field=models.CharField(blank=True, help_text='Eve Online name localized for German', max_length=100),
        ),
        migrations.AddField(
            model_name='structureservice',
            name='name_ko',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Korean', max_length=100),
        ),
        migrations.AddField(
            model_name='structureservice',
            name='name_ru',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Russian', max_length=100),
        ),
        migrations.AddField(
            model_name='structureservice',
            name='name_zh',
            field=models.CharField(blank=True, help_text='Eve Online name localized for Chinese', max_length=100),
        ),
        migrations.AddField(
            model_name='webhook',
            name='language_code',
            field=models.CharField(blank=True, choices=[('en', 'English'), ('de', 'German'), ('es', 'Spanish'), ('zh-hans', 'Chinese Simplified'), ('ru', 'Russian'), ('ko', 'Korean')], default=None, help_text='language of notifications send to this webhook', max_length=8, null=True, verbose_name='language'),
        ),
        migrations.AlterField(
            model_name='evecategory',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='evecategory',
            name='name',
            field=models.CharField(help_text='Eve Online name', max_length=100),
        ),
        migrations.AlterField(
            model_name='eveconstellation',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='eveconstellation',
            name='name',
            field=models.CharField(help_text='Eve Online name', max_length=100),
        ),
        migrations.AlterField(
            model_name='eveentity',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='evegroup',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='evegroup',
            name='name',
            field=models.CharField(help_text='Eve Online name', max_length=100),
        ),
        migrations.AlterField(
            model_name='evemoon',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='evemoon',
            name='name',
            field=models.CharField(help_text='Eve Online name', max_length=100),
        ),
        migrations.AlterField(
            model_name='evemoon',
            name='position_x',
            field=models.FloatField(blank=True, default=None, help_text='x position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='evemoon',
            name='position_y',
            field=models.FloatField(blank=True, default=None, help_text='y position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='evemoon',
            name='position_z',
            field=models.FloatField(blank=True, default=None, help_text='z position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='eveplanet',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='eveplanet',
            name='name',
            field=models.CharField(help_text='Eve Online name', max_length=100),
        ),
        migrations.AlterField(
            model_name='eveplanet',
            name='position_x',
            field=models.FloatField(blank=True, default=None, help_text='x position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='eveplanet',
            name='position_y',
            field=models.FloatField(blank=True, default=None, help_text='y position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='eveplanet',
            name='position_z',
            field=models.FloatField(blank=True, default=None, help_text='z position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='everegion',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='everegion',
            name='name',
            field=models.CharField(help_text='Eve Online name', max_length=100),
        ),
        migrations.AlterField(
            model_name='evesolarsystem',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='evesolarsystem',
            name='name',
            field=models.CharField(help_text='Eve Online name', max_length=100),
        ),
        migrations.AlterField(
            model_name='evetype',
            name='id',
            field=models.PositiveIntegerField(help_text='Eve Online ID', primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='evetype',
            name='name',
            field=models.CharField(help_text='Eve Online name', max_length=100),
        ),
        migrations.AlterField(
            model_name='notification',
            name='is_read',
            field=models.BooleanField(blank=True, default=None, help_text='True when this notification has read in the eve client', null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='text',
            field=models.TextField(blank=True, default=None, help_text='Notification details in YAML', null=True),
        ),
        migrations.AlterField(
            model_name='structure',
            name='next_reinforce_hour',
            field=models.PositiveIntegerField(blank=True, default=None, help_text='The requested change to reinforce_hour that will take effect at the time shown by next_reinforce_apply', null=True, validators=[django.core.validators.MaxValueValidator(23)]),
        ),
        migrations.AlterField(
            model_name='structure',
            name='next_reinforce_weekday',
            field=models.PositiveIntegerField(blank=True, default=None, help_text='The date and time when the structure’s newly requested reinforcement times (e.g. next_reinforce_hour and next_reinforce_day) will take effect', null=True, validators=[django.core.validators.MaxValueValidator(6)]),
        ),
        migrations.AlterField(
            model_name='structure',
            name='position_x',
            field=models.FloatField(blank=True, default=None, help_text='x position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='structure',
            name='position_y',
            field=models.FloatField(blank=True, default=None, help_text='y position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='structure',
            name='position_z',
            field=models.FloatField(blank=True, default=None, help_text='z position in the solar system', null=True),
        ),
        migrations.AlterField(
            model_name='structure',
            name='reinforce_hour',
            field=models.PositiveIntegerField(blank=True, default=None, help_text='The hour of day that determines the four hour window when the structure will randomly exit its reinforcement periods and become vulnerable to attack against its armor and/or hull. The structure will become vulnerable at a random time that is +/- 2 hours centered on the value of this property', null=True, validators=[django.core.validators.MaxValueValidator(23)]),
        ),
        migrations.AlterField(
            model_name='structure',
            name='reinforce_weekday',
            field=models.PositiveIntegerField(blank=True, default=None, help_text='(no longer used)', null=True, validators=[django.core.validators.MaxValueValidator(6)]),
        ),
        migrations.AlterField(
            model_name='structure',
            name='state',
            field=models.IntegerField(blank=True, choices=[(1, 'anchor vulnerable'), (2, 'anchoring'), (3, 'armor reinforce'), (4, 'armor vulnerable'), (5, 'deploy vulnerable'), (6, 'fitting invulnerable'), (7, 'hull reinforce'), (8, 'hull vulnerable'), (9, 'online deprecated'), (10, 'onlining vulnerable'), (11, 'shield vulnerable'), (12, 'unanchored'), (21, 'offline'), (22, 'online'), (23, 'onlining'), (24, 'reinforced'), (25, 'unanchoring '), (0, 'N/A'), (13, 'unknown')], default=13, help_text='Current state of the structure'),
        ),
        migrations.AlterField(
            model_name='structureservice',
            name='name',
            field=models.CharField(help_text='Name of the service', max_length=100),
        ),
        migrations.AlterField(
            model_name='structuretag',
            name='is_default',
            field=models.BooleanField(default=False, help_text='if true this tag will automatically be added to new structures'),
        ),
        migrations.AlterField(
            model_name='webhook',
            name='is_default',
            field=models.BooleanField(default=False, help_text='whether owners have this webhook automatically pre-set when created'),
        ),
    ]
